package swtpl.marsrover.odometry;

import swtpl.marsrover.net.RunContext;
import swtpl.marsrover.net.VehicleEvent;
import swtpl.marsrover.net.VehicleEvent.VehicleControlAcceleration;
import swtpl.marsrover.net.VehicleEvent.VehicleControlTurning;
import swtpl.marsrover.net.VehicleHandler;

public class Odometry implements VehicleHandler {
	/** acceleration in m/s^2 */
	double accel;
	/** brake acceleration (negative) m/s^2 */
	double brake;
	/** drag in 1/µm */
	double drag;
	
	boolean accelKnown;
	boolean brakeKnown;
	
	private final static int STEP = 10;
	
	int x, y, speed, dir, time;
	VehicleControlAcceleration accstate;
	
	public void calibrateAccel(int s1, int deltaS, int deltaT) {
		int ms = RunContext.getCurrentContext().getMaxSpeedMMPerSecs();
		accel = deltaS / ((double) deltaT * (1 - (double) s1*s1/ms/ms));
		drag = accel / (double) ms/ms;
	}
	
	public void calibrateBrake(int s1, int deltaS, int deltaT) {
		brake = (deltaS + drag*deltaT*(s1*s1)) / deltaT;
	}
	
	
	public void predictPosition(int deltaT, 
			VehicleControlAcceleration brkacc, 
			VehicleControlTurning turn) {
		double a;
		int angAdd;
		switch (brkacc) {
		case ACCELERATING:
			a = accel;
			break;
		case BRAKING:
			a = brake;
			break;
		case ROLLING:
		default:
			a = 0;
			break;
		}
		switch (turn) {
		case HARDRIGHT:
			angAdd = -RunContext.getCurrentContext().getTenthDegreePerSecsHardTurn();
			break;
		case RIGHT:
			angAdd = -RunContext.getCurrentContext().getTenthDegreePerSecsTurn();
			break;
		case LEFT:
			angAdd = RunContext.getCurrentContext().getTenthDegreePerSecsTurn();
			break;
		case HARDLEFT:
			angAdd = RunContext.getCurrentContext().getTenthDegreePerSecsHardTurn();
			break;
		case STRAIGHT:
		default:
			angAdd = 0;
			break;
		}
		double s = speed;
		double d = dir;
		double x = this.x;
		double y = this.y;
		for (int t = 0; t < deltaT; t+= STEP) {
			int step = deltaT - t;
			if (step > STEP)
				step = STEP;
			s += (int) (step * a - drag * step * s*s);
			if (s < 0)
				s = 0;
			x += Math.cos(d*2*Math.PI/3600) * s * step / 1000;
			y += Math.sin(d*2*Math.PI/3600) * s * step / 1000;
			d += step * angAdd / 1000.0;
			if (d > 1800)
				d -= 3600;
			if (d < -1800)
				d += 3600;
		}
		speed = (int) s;
		dir = (int) d;
		this.x = (int) x;
		this.y = (int) y;
	}

	public void changed(VehicleEvent ve) {
		if (!accelKnown) {
			if (accstate == VehicleControlAcceleration.ACCELERATING
				&& ve.getAccelerationState() == VehicleControlAcceleration.ACCELERATING) {
				System.err.println("MaxSpeed: "+RunContext.getCurrentContext().getMaxSpeedMMPerSecs());
				System.err.println("Speed: "+speed+ " dSpeed "+(ve.getSpeed() - speed)+ " dTime "+(ve.getTimeStamp()-time));
				calibrateAccel(speed, ve.getSpeed()- speed, ve.getTimeStamp() - time);
				System.err.println("Acceleration: "+accel +" Drag: "+drag);
				accelKnown = true;
			}	
		} else if (!brakeKnown 
				&& accstate == VehicleControlAcceleration.BRAKING) {
			if (ve.getAccelerationState() == VehicleControlAcceleration.BRAKING) {
				calibrateBrake(speed, ve.getSpeed()- speed, ve.getTimeStamp() - time);
				System.err.println("Brake: "+brake);
				brakeKnown = true;
			}
		} else {
			//predictPosition(ve.getTimeStamp() - time, accstate, ve.getTurningState());
			//int dx = ve.getPosX() - x;
			//int dy = ve.getPosY() - y;
			//int ds = ve.getSpeed() - speed;
			//int dd = ve.getDir() - dir;
			//System.err.println("dtime: " + (ve.getTimeStamp()-time) + 
			//		" dx: " + dx + " dy: "+dy+" ds: "+ ds+ " dd:"+dd);
		}
		
		x = ve.getPosX();
		y = ve.getPosY();
		dir = ve.getDir();
		speed = ve.getSpeed();
		time = ve.getTimeStamp();
		accstate = ve.getAccelerationState();
	}
	
	
}
