package swtpl.marsrover.planner;

import swtpl.marsrover.net.Connection;
import swtpl.marsrover.net.Connection.AccelerationCommand;
import swtpl.marsrover.net.Connection.TurningCommand;
import swtpl.marsrover.net.VehicleEvent.VehicleControlAcceleration;
import swtpl.marsrover.net.VehicleEvent.VehicleControlTurning;

public class Driver {
	int currAcc;
	int currTurn;
	Connection connection;
	
	public Driver(Connection conn) {
		connection = conn;
	}
	
	void reset() {
		currAcc = 0;
		currTurn = 0;
	}
	
	public void setDesiredStearing(int acc, int turn) {
		while (currAcc != acc || currTurn != turn) {
			AccelerationCommand acccmd = AccelerationCommand.NONE;
			TurningCommand turncmd = TurningCommand.NONE;
			if (currAcc < acc) {
				acccmd = AccelerationCommand.ACCELERATE;
				currAcc++;
			} else if (currAcc > acc) {
				acccmd = AccelerationCommand.BREAK;
				currAcc--;
			}
			if (currTurn < turn) {
				turncmd = TurningCommand.LEFT;
				currTurn++;
			} else if (currTurn > turn) {
				turncmd = TurningCommand.RIGHT;
				currTurn--;
			}
			connection.sendCommand(acccmd, turncmd);
		}
	}

	public void setState(VehicleControlAcceleration accelerationState,
			VehicleControlTurning turningState) {
		switch (accelerationState) {
		case BRAKING:
			currAcc = -1;
			break;
		case ROLLING:
			currAcc = 0;
			break;
		case ACCELERATING:
			currAcc = 1;
			break;
		}
		switch (turningState) {
		case HARDRIGHT:
			currTurn = -2;
			break;
		case RIGHT:
			currTurn = -1;
			break;
		case LEFT:
			currTurn = 1;
			break;
		case HARDLEFT:
			currTurn = 2;
			break;
		case STRAIGHT:
			currTurn = 0;
			break;
		}
		
	}
}
